@@ -1,5 +1,5 @@
-// Avisynth C Interface Version 0.20 stdcall
-// Copyright 2003 Kevin Atkinson
+// Avisynth C Interface
+// Based on Copyright 2003 Kevin Atkinson
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
 #include <avisynth.h>
 #include <avisynth_c.h>
 #include "AVSMap.h"
+#include "internal.h"
 
 #ifdef AVS_WINDOWS
 #include <avs/win.h>
@@ -254,88 +255,44 @@ int AVSC_CC avs_bmp_size(const AVS_VideoInfo * vi)
 extern "C"
 int AVSC_CC avs_get_pitch_p(const AVS_VideoFrame * p, int plane)
 {
-  switch (plane) {
-  case AVS_PLANAR_U: case AVS_PLANAR_V: return p->pitchUV;
-  case AVS_PLANAR_A: return p->pitchA;
-  }
-  return p->pitch; // Y, G, B, R
+  // Memo: the lines in class PVideoFrame:
+  //   VideoFrame* p;
+  //   VideoFrame* operator->() const { return p; }
+  // help when you use the arrow operator on a PVideoFrame object, it will return 
+  // the "VideoFrame* p" pointer, allowing you to access members of the 
+  // VideoFrame class directly.
+  return (*(const PVideoFrame*)&p)->GetPitch(plane);
 }
 
 extern "C"
 int AVSC_CC avs_get_row_size_p(const AVS_VideoFrame * p, int plane)
 {
-  int r;
-
-  switch (plane) {
-  case AVS_PLANAR_U: case AVS_PLANAR_V:
-    return (p->pitchUV) ? p->row_sizeUV : 0;
-
-  case AVS_PLANAR_U_ALIGNED: case AVS_PLANAR_V_ALIGNED:
-    if (p->pitchUV) {
-      r = (p->row_sizeUV + FRAME_ALIGN - 1) & (~(FRAME_ALIGN - 1)); // Aligned rowsize
-      return (r <= p->pitchUV) ? r : p->row_sizeUV;
-    }
-    else
-      return 0;
-
-  case AVS_PLANAR_ALIGNED: case AVS_PLANAR_Y_ALIGNED:
-  case AVS_PLANAR_R_ALIGNED: case AVS_PLANAR_G_ALIGNED: case AVS_PLANAR_B_ALIGNED:
-    r = (p->row_size + FRAME_ALIGN - 1) & (~(FRAME_ALIGN - 1)); // Aligned rowsize
-    return (r <= p->pitch) ? r : p->row_size;
-  case AVS_PLANAR_A:
-    return (p->pitchA) ? p->row_sizeA : 0;
-  case AVS_PLANAR_A_ALIGNED:
-    if (p->pitchA) {
-      r = (p->row_sizeA + FRAME_ALIGN - 1) & (~(FRAME_ALIGN - 1)); // Aligned rowsize
-      return (r <= p->pitchA) ? r : p->row_sizeA;
-    }
-    else
-      return 0;
-  }
-  return p->row_size;
+  return (*(const PVideoFrame*)&p)->GetRowSize(plane);
 }
 
 extern "C"
 int AVSC_CC avs_get_height_p(const AVS_VideoFrame * p, int plane)
 {
-  switch (plane) {
-  case AVS_PLANAR_U: case AVS_PLANAR_V:
-    return (p->pitchUV) ? p->heightUV : 0;
-  case AVS_PLANAR_A:
-    return (p->pitchA) ? p->height : 0;
-  }
-  return p->height; // Y, G, B, R, A
+  return (*(const PVideoFrame*)&p)->GetHeight(plane);
 }
 
 extern "C"
 const BYTE * AVSC_CC avs_get_read_ptr_p(const AVS_VideoFrame * p, int plane)
 {
-  switch (plane) {
-  case AVS_PLANAR_U: case AVS_PLANAR_B: return p->vfb->data + p->offsetU; // G is first. Then B,R order like U,V
-  case AVS_PLANAR_V: case AVS_PLANAR_R: return p->vfb->data + p->offsetV;
-  case AVS_PLANAR_A: return p->vfb->data + p->offsetA;
-  default:           return p->vfb->data + p->offset;
-  } // AVS_PLANAR Y, AVS_PLANAR_G
+  return (*(const PVideoFrame*)&p)->GetReadPtr(plane);
 }
 
 extern "C"
 int AVSC_CC avs_is_writable(const AVS_VideoFrame * p)
 {
-  if (p->refcount == 1 && p->vfb->refcount == 1) {
-    InterlockedIncrement(&(p->vfb->sequence_number));
-    return 1;
-  }
-  return 0;
+  return (*(const PVideoFrame*)&p)->IsWritable() ? 1 : 0;
 }
 
 // V9
 extern "C"
 int AVSC_CC avs_is_property_writable(const AVS_VideoFrame * p)
 {
-  if (p->refcount == 1) {
-    return 1;
-  }
-  return 0;
+  return (*(const PVideoFrame*)&p)->IsPropertyWritable() ? 1 : 0;
 }
 
 // V10
@@ -354,16 +311,7 @@ void AVSC_CC avs_video_frame_amend_pixel_type(AVS_VideoFrame* p, int new_pixel_t
 extern "C"
 BYTE * AVSC_CC avs_get_write_ptr_p(const AVS_VideoFrame * p, int plane)
 {
-  switch (plane) {
-  case AVS_PLANAR_U: case AVS_PLANAR_B: return p->vfb->data + p->offsetU;
-  case AVS_PLANAR_V: case AVS_PLANAR_R: return p->vfb->data + p->offsetV;
-  case AVS_PLANAR_A: return p->vfb->data + p->offsetA;
-  default:           break;
-  }
-  if (avs_is_writable(p)) {
-    return p->vfb->data + p->offset; // Y,G
-  }
-  return 0;
+  return (*(const PVideoFrame*)&p)->GetWritePtr(plane);
 }
 
 extern "C"
@@ -539,6 +487,19 @@ int64_t AVSC_CC avs_prop_get_int(AVS_ScriptEnvironment * p, const AVS_Map * map,
   }
 }
 
+extern "C"
+int AVSC_CC avs_prop_get_int_saturated(AVS_ScriptEnvironment* p, const AVS_Map* map, const char* key, int index, int* error)
+{
+  p->error = 0;
+  try {
+    return (p->env->propGetIntSaturated((const AVSMap*)map, key, index, error));
+  }
+  catch (const AvisynthError& err) {
+    p->error = err.msg;
+    return 0;
+  }
+}
+
 extern "C"
 double AVSC_CC avs_prop_get_float(AVS_ScriptEnvironment * p, const AVS_Map * map, const char* key, int index, int* error)
 {
@@ -552,6 +513,20 @@ double AVSC_CC avs_prop_get_float(AVS_ScriptEnvironment * p, const AVS_Map * map
   }
 }
 
+// v11
+extern "C"
+float AVSC_CC avs_prop_get_float_saturated(AVS_ScriptEnvironment* p, const AVS_Map* map, const char* key, int index, int* error)
+{
+  p->error = 0;
+  try {
+    return (p->env->propGetFloatSaturated((const AVSMap*)map, key, index, error));
+  }
+  catch (const AvisynthError& err) {
+    p->error = err.msg;
+    return 0;
+  }
+}
+
 extern "C"
 const char* AVSC_CC avs_prop_get_data(AVS_ScriptEnvironment * p, const AVS_Map * map, const char* key, int index, int* error)
 {
@@ -582,6 +557,19 @@ int AVSC_CC avs_prop_get_data_size(AVS_ScriptEnvironment * p, const AVS_Map * ma
   }
 }
 
+extern "C"
+int AVSC_CC avs_prop_get_data_type_hint(AVS_ScriptEnvironment* p, const AVS_Map* map, const char* key, int index, int* error)
+{
+  p->error = 0;
+  try {
+    return (p->env->propGetDataTypeHint((const AVSMap*)map, key, index, error));
+  }
+  catch (const AvisynthError& err) {
+    p->error = err.msg;
+    return 0;
+  }
+}
+
 extern "C"
 AVS_Clip* AVSC_CC avs_prop_get_clip(AVS_ScriptEnvironment* p, const AVS_Map* map, const char* key, int index, int* error)
 {
@@ -655,10 +643,23 @@ int AVSC_CC avs_prop_set_data(AVS_ScriptEnvironment * p, AVS_Map * map, const ch
 }
 
 extern "C"
-int AVSC_CC avs_prop_set_clip(AVS_ScriptEnvironment * p, AVS_Map * map, const char* key, AVS_Clip * clip, int append)
+int AVSC_CC avs_prop_set_data_h(AVS_ScriptEnvironment* p, AVS_Map* map, const char* key, const char* d, int length, int type, int append)
 {
   // length = -1 -> auto strlen
   p->error = 0;
+  try {
+    return (p->env->propSetDataH((AVSMap*)map, key, d, length, type, append));
+  }
+  catch (const AvisynthError& err) {
+    p->error = err.msg;
+    return 0;
+  }
+}
+
+extern "C"
+int AVSC_CC avs_prop_set_clip(AVS_ScriptEnvironment * p, AVS_Map * map, const char* key, AVS_Clip * clip, int append)
+{
+   p->error = 0;
   try {
     return (p->env->propSetClip((AVSMap*)map, key, *(PClip*)clip, append));
   }
@@ -671,7 +672,6 @@ int AVSC_CC avs_prop_set_clip(AVS_ScriptEnvironment * p, AVS_Map * map, const ch
 extern "C"
 int AVSC_CC avs_prop_set_frame(AVS_ScriptEnvironment * p, AVS_Map * map, const char* key, const AVS_VideoFrame * frame, int append)
 {
-  // length = -1 -> auto strlen
   p->error = 0;
   try {
     return (p->env->propSetFrame((AVSMap*)map, key, *(PVideoFrame*)&frame, append));
@@ -927,30 +927,146 @@ AVS_Clip * AVSC_CC avs_take_clip(AVS_Value v, AVS_ScriptEnvironment * env)
   return c;
 }
 
+// v11 API AVS_Value type checkers.
 extern "C"
-void AVSC_CC avs_set_to_clip(AVS_Value * v, AVS_Clip * c)
-{
-  new(v) AVSValue(c->clip);
-}
+int AVSC_CC avs_val_defined(AVS_Value v) { return ((const AVSValue*)(&v))->Defined() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_clip(AVS_Value v) { return ((const AVSValue*)(&v))->IsClip() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_bool(AVS_Value v) { return ((const AVSValue*)(&v))->IsBool() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_int(AVS_Value v) { return ((const AVSValue*)(&v))->IsInt() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_long_strict(AVS_Value v) { return ((const AVSValue*)(&v))->IsLongStrict() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_float(AVS_Value v) { return ((const AVSValue*)(&v))->IsFloat() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_floatf_strict(AVS_Value v) { return ((const AVSValue*)(&v))->IsFloatfStrict() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_string(AVS_Value v) { return ((const AVSValue*)(&v))->IsString() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_array(AVS_Value v) { return ((const AVSValue*)(&v))->IsArray() ? 1 : 0; }
+extern "C"
+int AVSC_CC avs_val_is_error(AVS_Value v) { return v.type == 'e' ? 1 : 0; }
+
+
+// v11 API AVS_Value setters
+extern "C"
+void AVSC_CC avs_set_to_error(AVS_Value* v, const char* v0) { new(v) AVSValue(v0); /*string->error*/ v->type = 'e'; }
+extern "C"
+void AVSC_CC avs_set_to_bool(AVS_Value* v, int v0) { new(v) AVSValue(v0 != 0); }
+extern "C"
+void AVSC_CC avs_set_to_int(AVS_Value* v, int v0) { new(v) AVSValue(v0); }
+extern "C"
+void AVSC_CC avs_set_to_float(AVS_Value* v, float v0) { new(v) AVSValue(v0); }
+extern "C"
+void AVSC_CC avs_set_to_string(AVS_Value* v, const char* v0) { new(v) AVSValue(v0); }
+extern "C"
+void AVSC_CC avs_set_to_double(AVS_Value* v, double d) { new(v) AVSValue(d); }
+extern "C"
+void AVSC_CC avs_set_to_long(AVS_Value* v, int64_t l) { new(v) AVSValue(l); }
+extern "C"
+void AVSC_CC avs_set_to_array(AVS_Value* v, AVS_Value* src, int size) { new(v) AVSValue((AVSValue *)src, size); }
+extern "C"
+void AVSC_CC avs_set_to_void(AVS_Value* v) { new(v) AVSValue(); }
+// existed pre V11
+extern "C"
+void AVSC_CC avs_set_to_clip(AVS_Value* v, AVS_Clip* c) { new(v) AVSValue(c->clip); }
 
 extern "C"
 void AVSC_CC avs_copy_value(AVS_Value * dest, AVS_Value src)
 {
+#if 0
+  // no need to guard multidim arrays. avs_release_value will release properly.
   // true: don't copy array elements recursively
   new(dest) AVSValue(*(const AVSValue*)&src, true);
+#endif
+  // CONSTRUCTOR9-->Assign(&v, init=true) ensures that the original content
+  // is simply overwritten and will not be freed.
+  new(dest) AVSValue(*(const AVSValue*)&src);
 }
 
+// Releases/free resources contained in an AVS_Value
+// Such types are: 
+// - clip
+// - function (not supported on C interface)
+// - double and long on 32 bit architects
+// - Avisynth+ arrays
+// Since AVS_Value is just a struct passed by value, nothing else happens.
+// If AVS_Value has no extra resource to free up, nothing is done.
+// release is mandatory for results of:
+// - avs_invoke 
+// - avs_copy_value 
+// - avs_set_to_double (x86)
+// - avs_set_to_long (x86)
+// - avs_set_to_array, avs_set_to_clip
+// - in general: all avs_set_xxx values
+
+// Note:
+// Unlike dynamic arrays, which have a full create-copy-modify-release cycle,
+// C plugins/clients can use in-source arrays, typically for assembling function
+// parameters ("args"). Do not call avs_release_value on such arrays, as it causes
+// a crash since the array itself and the elements are not maintained by the 
+// Avisynth core.However, it is good practice to release the array content
+// with avs_release_value one-by-one for the resource-allocated types mentioned above.
+
 extern "C"
 void AVSC_CC avs_release_value(AVS_Value v)
 {
+#if 0
+  // This would prevent crashes when a C client calls it for an array.
+  // However, since Avisynth+ arrays can be returned by avs_invoke, avs_copy_value,
+  // and avs_set_to_array, this exemption is no longer valid.
+  // This rule was not explicitly written earlier and must be enforced in clients/plugins,
+  // see the Python wrapper issue in AvsPmod, which was fixed.
   if (((AVSValue*)&v)->IsArray()) {
     // signing for destructor: don't free array elements
-    ((AVSValue*)&v)->MarkArrayAsC();
+    ((AVSValue*)&v)->MarkArrayAsNonDeepCopy();
   }
-
+#endif  
   ((AVSValue*)&v)->~AVSValue();
 }
 
+// API AVS_Value getters, like INLINE versions
+extern "C"
+int AVSC_CC avs_get_as_bool(AVS_Value v) { return ((AVSValue*)&v)->AsBool() ? 1 : 0; }
+
+extern "C"
+AVS_Clip* AVSC_CC avs_get_as_clip(AVS_Value v, AVS_ScriptEnvironment* env) {
+  // like the existing avs_take_clip, to fit in the avs_get_as_xxxx line
+  AVS_Clip* c = new AVS_Clip;
+  c->env = env->env;
+  c->clip = (IClip*)(v.d.clip);
+  return c;
+}
+
+extern "C"
+int AVSC_CC avs_get_as_int(AVS_Value v) { return ((AVSValue*)&v)->AsInt(); }
+extern "C"
+int64_t AVSC_CC avs_get_as_long(AVS_Value v) { return ((AVSValue*)&v)->AsLong(); }
+extern "C"
+const char* AVSC_CC avs_get_as_string(AVS_Value v) { return ((AVSValue*)&v)->AsString(); }
+extern "C"
+double AVSC_CC avs_get_as_float(AVS_Value v) { return ((AVSValue*)&v)->AsFloat(); }
+extern "C"
+const char* AVSC_CC avs_get_as_error(AVS_Value v) {
+  if (v.type == 'e') {
+    v.type = 's'; // 'e'rror is unknown in c++ api
+    return ((AVSValue*)&v)->AsString();
+  }
+  return nullptr;
+}
+extern "C"
+const AVS_Value * AVSC_CC avs_get_as_array(AVS_Value v) { return v.d.array; }
+extern "C"
+AVS_Value AVSC_CC avs_get_array_elt(AVS_Value v, int index) 
+{ // just a dumb data copy, no ref counting, no resource handling
+  return avs_is_array(v) ? v.d.array[index] : v;
+}
+extern "C"
+int AVSC_CC avs_get_array_size(AVS_Value v) { return ((AVSValue*)&v)->ArraySize(); }
+
+
 //////////////////////////////////////////////////////////////////
 //
 //
@@ -963,7 +1079,7 @@ AVS_Clip * AVSC_CC avs_new_c_filter(AVS_ScriptEnvironment * e,
 {
   C_VideoFilter* f = new C_VideoFilter();
   AVS_Clip* ff = new AVS_Clip();
-  ff->clip = f;
+  ff->clip = f; // IClip descendant
   ff->env = e->env;
   f->env.env = e->env;
   f->d.env = &f->env;
@@ -974,6 +1090,10 @@ AVS_Clip * AVSC_CC avs_new_c_filter(AVS_ScriptEnvironment * e,
     f->d.child = &f->child;
   }
   *fi = &f->d;
+  // (*fi)->free_filter will be set later by the plugin.
+  // It is used in ~C_VideoFilter, which is called when the clip's 
+  // reference count reaches zero and it is released. This serves 
+  // as the filter destructor in C.
   if (child.type == 'c')
     f->d.vi = *(const AVS_VideoInfo*)(&((IClip*)child.d.clip)->GetVideoInfo());
   return ff;
@@ -987,6 +1107,7 @@ AVS_Clip * AVSC_CC avs_new_c_filter(AVS_ScriptEnvironment * e,
 struct C_VideoFilter_UserData {
   void* user_data;
   AVS_ApplyFunc func;
+  AVS_ApplyFuncR func_r;
 };
 
 AVSValue __cdecl create_c_video_filter(AVSValue args, void* user_data,
@@ -997,8 +1118,11 @@ AVSValue __cdecl create_c_video_filter(AVSValue args, void* user_data,
   env.env = e0;
   env.error = NULL;
 
-  //	OutputDebugString("OK");
-  AVS_Value res = (d->func)(&env, *(AVS_Value*)&args, d->user_data);
+  AVS_Value res;
+  if(d->func)
+    res = (d->func)(&env, *(AVS_Value*)&args, d->user_data);
+  else
+    (d->func_r)(&env, (AVS_Value*)&res, *(AVS_Value*)&args, d->user_data); // new in v11: byref
   if (res.type == 'e') {
     throw AvisynthError(res.d.string);
   }
@@ -1017,7 +1141,31 @@ avs_add_function(AVS_ScriptEnvironment * p, const char* name, const char* params
 {
   C_VideoFilter_UserData* dd, * d = new C_VideoFilter_UserData;
   p->error = 0;
-  d->func = applyf;
+  d->func = applyf; // return value struct
+  d->func_r = nullptr;
+  d->user_data = user_data;
+  dd = (C_VideoFilter_UserData*)p->env->SaveString((const char*)d, sizeof(C_VideoFilter_UserData));
+  delete d;
+  try {
+    p->env->AddFunction(name, params, create_c_video_filter, dd);
+  }
+  catch (AvisynthError& err) {
+    p->error = err.msg;
+    return -1;
+  }
+  return 0;
+}
+
+// v11
+extern "C"
+int AVSC_CC
+avs_add_function_r(AVS_ScriptEnvironment* p, const char* name, const char* params,
+  AVS_ApplyFuncR applyf, void* user_data)
+{
+  C_VideoFilter_UserData* dd, * d = new C_VideoFilter_UserData;
+  p->error = 0;
+  d->func = nullptr;
+  d->func_r = applyf;  // return value among parameters byref struct (Python cref callbacks like it better)
   d->user_data = user_data;
   dd = (C_VideoFilter_UserData*)p->env->SaveString((const char*)d, sizeof(C_VideoFilter_UserData));
   delete d;
@@ -1083,11 +1231,16 @@ int AVSC_CC avs_function_exists(AVS_ScriptEnvironment * p, const char* name)
 }
 
 extern "C"
-AVS_Value AVSC_CC avs_invoke(AVS_ScriptEnvironment * p, const char* name, AVS_Value args, const char** arg_names)
+AVS_Value AVSC_CC avs_invoke(AVS_ScriptEnvironment* p, const char* name, AVS_Value args, const char** arg_names)
 {
   AVS_Value v = { 0,0 };
   p->error = 0;
   try {
+    // AVSValue(*(AVSValue*)&args) is deep-copying input array
+
+    // Invoke has a special InvokePreV11C variant if pre V11 C interface 
+    // detected, which converts a possible 64 bit result to 32 bit one for
+    // compatibility. double->float, long->int. Nothing should be done here.
     AVSValue v0 = p->env->Invoke(name, *(AVSValue*)&args, arg_names);
     new ((AVSValue*)&v) AVSValue(v0);
   }
@@ -1460,6 +1613,9 @@ int64_t AVSC_CC avs_get_var_long(AVS_ScriptEnvironment * p, const char* name, in
 //
 //
 
+// prototype from avisynth.cpp
+IScriptEnvironment2* CreateScriptEnvironment2_internal(int version, bool fromAvs25, bool fromC);
+
 extern "C"
 AVS_ScriptEnvironment * AVSC_CC avs_create_script_environment(int version)
 {
@@ -1467,7 +1623,7 @@ AVS_ScriptEnvironment * AVSC_CC avs_create_script_environment(int version)
   try {
     if (version < AVISYNTH_CLASSIC_INTERFACE_VERSION)
       version = AVISYNTH_CLASSIC_INTERFACE_VERSION; // always request a more modern ScriptEnvironment.
-    e->env = CreateScriptEnvironment(version);
+    e->env = CreateScriptEnvironment2_internal(version, false, true); // flag: from C
     e->error = NULL;
   }
   catch (const AvisynthError& err) {
