@@ -1,5 +1,5 @@
 // Avisynth v2.6.  Copyright 2002-2009 Ben Rudiak-Gould et al.
-// http://www.avisynth.org
+// http://avisynth.nl
 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include "InternalEnvironment.h"
 #include "DeviceManager.h"
 #include <cassert>
+#include <chrono>
 #include <cstdio>
 
 #ifdef X86_32
@@ -83,10 +84,10 @@ struct CachePimpl
   // Audio cache
   CachePolicyHint AudioPolicy;
   char* AudioCache;
-  size_t SampleSize;
-  size_t MaxSampleCount;
+  int SampleSize;
+  int64_t MaxSampleCount;
   int64_t AudioCacheStart;
-  size_t CacheCount;
+  int64_t CacheCount;
   int64_t ac_expected_next;
   int ac_too_small_count;
   long ac_currentscore;
@@ -128,12 +129,13 @@ Cache::Cache(const PClip& _child, Device* device, std::mutex& CacheGuardMutex, I
 
   const int dummy = 0;
   // child is usually MTGuard, which forwards this request to its child filter - the real one
-  CachePolicyHint childAudioPolicy = (CachePolicyHint)_child->SetCacheHints(CACHE_GETCHILD_AUDIO_MODE, dummy);
+  const bool overAvs25 = (_child->GetVersion() >= 5);
+  CachePolicyHint childAudioPolicy = (CachePolicyHint)(overAvs25 ? _child->SetCacheHints(CACHE_GETCHILD_AUDIO_MODE, dummy) : 0);
   // if not set by the plugin, get default (set in CachePimpl ctor)
   if (childAudioPolicy == 0)
     childAudioPolicy = (CachePolicyHint)this->SetCacheHints(CACHE_GET_AUDIO_POLICY, dummy);
   switch (childAudioPolicy) {
-  case 0: break; // n/a
+  case CachePolicyHint(0): break; // n/a
   case CACHE_AUDIO:          // Explicitly do cache audio, X byte cache.
   case CACHE_AUDIO_NOTHING:  // Explicitly do not cache audio.
   case CACHE_AUDIO_AUTO_START_OFF: // Audio cache off (auto mode), X byte initial cache.
@@ -141,7 +143,7 @@ Cache::Cache(const PClip& _child, Device* device, std::mutex& CacheGuardMutex, I
     // ask child for desired audio cache size
     // e.g. EnsureVBRMP3Sync explicitely requests CACHE_AUDIO and 1024*1024
     // This will create cache only if clip has audio
-    this->SetCacheHints(childAudioPolicy, _child->SetCacheHints(CACHE_GETCHILD_AUDIO_SIZE, dummy)); // if size=0 remains the default
+    this->SetCacheHints(childAudioPolicy, overAvs25 ? _child->SetCacheHints(CACHE_GETCHILD_AUDIO_SIZE, dummy) : 0); // if size=0 remains the default
     break;
   default:
     env->ThrowError("Cache: Filter returned invalid response to CACHE_GETCHILD_AUDIO_MODE. %d", childAudioPolicy);
@@ -162,7 +164,7 @@ PVideoFrame __stdcall Cache::GetFrame(int n, IScriptEnvironment* env_)
   std::unique_ptr<char[]> buf(new char[BUFSIZE+1]);
 
 #endif
-  InternalEnvironment *env = static_cast<InternalEnvironment*>(env_);
+  InternalEnvironment* env = GetAndRevealCamouflagedEnv(env_);
 
   // Protect plugins that cannot handle out-of-bounds frame indices
   n = clamp(n, 0, GetVideoInfo().num_frames-1);
@@ -291,7 +293,7 @@ PVideoFrame __stdcall Cache::GetFrame(int n, IScriptEnvironment* env_)
   return result;
 }
 
-void Cache::FillAudioZeros(void* buf, int start_offset, int count) {
+void Cache::FillAudioZeros(void* buf, size_t start_offset, size_t count) {
     const int bps = _pimpl->vi.BytesPerAudioSample();
     unsigned char* byte_buf = (unsigned char*)buf;
     memset(byte_buf + start_offset * bps, 0, count * bps);
@@ -301,6 +303,8 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
 {
   if (count <= 0)
     return;
+  if (count > 0 && static_cast<uint64_t>(count) > std::numeric_limits<size_t>::max())
+    env->ThrowError("GetAudio error: Count exceeds platform maximum buffer size limit.");
 
   // -----------------------------------------------------------
   //          Enforce audio bounds
@@ -310,20 +314,20 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
 
   if ((!vi->HasAudio()) || (start + count <= 0) || (start >= vi->num_audio_samples)) {
     // Completely skip.
-    FillAudioZeros(buf, 0, (int)count);
+    FillAudioZeros(buf, 0, (size_t)count);
     count = 0;
     return;
   }
 
   if (start < 0) {  // Partial initial skip
-    FillAudioZeros(buf, 0, (int)-start);  // Fill all samples before 0 with silence.
+    FillAudioZeros(buf, 0, static_cast<size_t>(-start));  // Fill all samples before 0 with silence.
     count += start;  // Subtract start bytes from count.
-    buf = ((BYTE*)buf) - (int)(start*vi->BytesPerAudioSample());
+    buf = ((BYTE*)buf) - start * vi->BytesPerAudioSample();
     start = 0;
   }
 
   if (start + count > vi->num_audio_samples) {  // Partial ending skip
-    FillAudioZeros(buf, (int)(vi->num_audio_samples - start), (int)(count - (vi->num_audio_samples - start)));  // Fill end samples
+    FillAudioZeros(buf, (size_t)(vi->num_audio_samples - start), (size_t)(count - (vi->num_audio_samples - start)));  // Fill end samples
     count = (vi->num_audio_samples - start);
   }
 
@@ -346,10 +350,11 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
 
   // Change from AUTO_OFF to AUTO_ON
   if (_pimpl->AudioPolicy == CACHE_AUDIO_AUTO_START_OFF && _pimpl->ac_currentscore <= 0) {
-    int new_size = (int)(_pimpl->vi.BytesFromAudioSamples(count) + 8191) & -8192;
-    new_size = min(4096 * 1024, new_size);
-    _RPT2(0, "CA:%x: Automatically adding %d byte audiocache!\n", this, new_size);
-    SetCacheHints(CACHE_AUDIO_AUTO_START_ON, new_size);
+    // align size to 8192 byte boundary
+    int64_t new_size = (_pimpl->vi.BytesFromAudioSamples(count) + 8191) & -8192;
+    new_size = min((int64_t)4096 * 1024, new_size);
+    _RPT2(0, "CA:%x: Automatically adding %d byte audiocache!\n", this, (int)new_size);
+    SetCacheHints(CACHE_AUDIO_AUTO_START_ON, (int)new_size);
   }
 
   // Change from AUTO_ON to AUTO_OFF
@@ -375,10 +380,10 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
 
     if (_pimpl->ac_too_small_count > 2 && _pimpl->MaxSampleCount < _pimpl->vi.AudioSamplesFromBytes(8192 * 1024)) {  // Max size = 8MB!
       //automatically upsize cache!
-      int new_size = (int)(_pimpl->vi.BytesFromAudioSamples(std::max(count, _pimpl->AudioCacheStart + (int64_t)_pimpl->CacheCount - start)) + 8192) & -8192; // Yes +1 to +8192 bytes
-      new_size = std::min(8192 * 1024, new_size);
-      _RPT2(0, "CA:%x: Autoupsizing buffer to %d bytes!\n", this, new_size);
-      SetCacheHints(_pimpl->AudioPolicy, new_size); // updates maxsamplecount!!
+      int64_t new_size = (_pimpl->vi.BytesFromAudioSamples(std::max(count, _pimpl->AudioCacheStart + _pimpl->CacheCount - start)) + 8192) & -8192; // Yes +1 to +8192 bytes
+      new_size = std::min((int64_t)8192 * 1024, new_size);
+      _RPT2(0, "CA:%x: Autoupsizing buffer to %d bytes!\n", this, (int)new_size);
+      SetCacheHints(_pimpl->AudioPolicy, (int)new_size); // updates maxsamplecount!!
       _pimpl->ac_too_small_count = 0;
     }
     else {
@@ -386,7 +391,7 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
       _pimpl->child->GetAudio(buf, start, count, env);
       global_lock.lock();
 
-      _pimpl->CacheCount = std::min((size_t)count, _pimpl->MaxSampleCount); // Remember maxsamplecount gets updated
+      _pimpl->CacheCount = std::min(count, _pimpl->MaxSampleCount); // Remember maxsamplecount gets updated
       _pimpl->AudioCacheStart = start + count - _pimpl->CacheCount;
       BYTE* buff = (BYTE*)buf;
       buff += _pimpl->vi.BytesFromAudioSamples(_pimpl->AudioCacheStart - start);
@@ -401,7 +406,7 @@ void __stdcall Cache::GetAudio(void* buf, int64_t start, int64_t count, IScriptE
   if ((start < _pimpl->AudioCacheStart) || (start > _pimpl->AudioCacheStart + _pimpl->MaxSampleCount)) { //first sample is before cache or beyond linear reach -> restart cache
     _RPT1(0, "CA:%x: Restart\n", this);
 
-    _pimpl->CacheCount = std::min((size_t)count, _pimpl->MaxSampleCount);
+    _pimpl->CacheCount = std::min(count, _pimpl->MaxSampleCount);
     _pimpl->AudioCacheStart = start;
     _pimpl->child->GetAudio(_pimpl->AudioCache, _pimpl->AudioCacheStart, _pimpl->CacheCount, env);
   }
@@ -620,7 +625,7 @@ PClip CacheGuard::GetCache(IScriptEnvironment* env_)
 {
   std::unique_lock<std::mutex> global_lock(mutex);
 
-  InternalEnvironment* env = static_cast<InternalEnvironment*>(env_);
+  InternalEnvironment* env = GetAndRevealCamouflagedEnv(env_);
 
   Device* device = env->GetCurrentDevice();
 
@@ -663,41 +668,26 @@ int CacheGuard::GetOrDefault(int cachehints, int frame_range, int def)
   return def;
 }
 
-PVideoFrame __stdcall CacheGuard::GetFrame(int n, IScriptEnvironment* env)
+PVideoFrame __stdcall CacheGuard::GetFrame(int n, IScriptEnvironment* env_)
 {
-  InternalEnvironment* IEnv;
-  // When GetFrame is called from an Avs Cpp 2.5 plugin constructor,
-  // 'env' is a disguised IScriptEnvirontment_Avs25 which we cannot
-  // static cast to InternalEnvironment directly.
-  // We have to figure out whether the environment is v2.5 and act upon.
-  if (env->ManageCache((int)MC_QueryAvs25, nullptr) == (intptr_t*)1) {
-    IEnv = static_cast<InternalEnvironment*>(reinterpret_cast<IScriptEnvironment_Avs25*>(env));
-  }
-  else {
-    IEnv = static_cast<InternalEnvironment*>(env);
-  }
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   ScopedCounter getframe_counter(IEnv->GetFrameRecursiveCount());
-  IScriptEnvironment* env_real = static_cast<IScriptEnvironment*>(IEnv);
   /*
   if (!name.empty())
     _RPT2(0, "CacheGuard::GetFrame call further GetFrame: %s %d\n", name.c_str(), n);
   */
-  return GetCache(env_real)->GetFrame(n, env_real);
+  return GetCache(env)->GetFrame(n, env);
 }
 
-void __stdcall CacheGuard::GetAudio(void* buf, int64_t start, int64_t count, IScriptEnvironment* env)
+void __stdcall CacheGuard::GetAudio(void* buf, int64_t start, int64_t count, IScriptEnvironment* env_)
 {
-  InternalEnvironment* IEnv;
-  // see Avs2.5 comments on CacheGuard::GetFrame
-  if (env->ManageCache((int)MC_QueryAvs25, nullptr) == (intptr_t*)1) {
-    IEnv = static_cast<InternalEnvironment*>(reinterpret_cast<IScriptEnvironment_Avs25*>(env));
-  }
-  else {
-    IEnv = static_cast<InternalEnvironment*>(env);
-  }
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   ScopedCounter getframe_counter(IEnv->GetFrameRecursiveCount());
-  IScriptEnvironment* env_real = static_cast<IScriptEnvironment*>(IEnv);
-  return GetCache(env_real)->GetAudio(buf, start, count, env_real);
+  return GetCache(env)->GetAudio(buf, start, count, env);
 }
 
 const VideoInfo& __stdcall CacheGuard::GetVideoInfo()
@@ -810,7 +800,7 @@ int __stdcall CacheGuard::SetCacheHints(int cachehints, int frame_range)
     *********************************************/
   case CACHE_GETCHILD_AUDIO_MODE:
   case CACHE_GETCHILD_AUDIO_SIZE:
-    return child->SetCacheHints(cachehints, 0);
+    return (child->GetVersion() >= 5) ? child->SetCacheHints(cachehints, 0) : 0;
 
   case CACHE_AUDIO:
   case CACHE_AUDIO_AUTO_START_ON: // auto mode, initially cache
@@ -825,7 +815,7 @@ int __stdcall CacheGuard::SetCacheHints(int cachehints, int frame_range)
     return hints.AudioPolicy;
 
   case CACHE_GET_AUDIO_SIZE: // Get the current audio cache size.
-    return GetOrDefault(cachehints, frame_range, 0);
+    return (child->GetVersion() >= 5) ? GetOrDefault(cachehints, frame_range, 0) : 0;
 
   case CACHE_PREFETCH_AUDIO_BEGIN:    // Begin queue request to prefetch audio (take critical section).
   case CACHE_PREFETCH_AUDIO_STARTLO:  // Set low 32 bits of start.
