@@ -1,6 +1,6 @@
 
 // Avisynth v2.5.  Copyright 2002 Ben Rudiak-Gould et al.
-// http://www.avisynth.org
+// http://avisynth.nl
 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -920,7 +920,10 @@ AVSValue MinMaxPlane::MinMax(AVSValue clip, void* , double threshold, int offset
 }
 
 
-AVSValue GetProperty::Create(AVSValue args, void* user_data, IScriptEnvironment* env) {
+AVSValue GetProperty::Create(AVSValue args, void* user_data, IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propGetxxxxx: No clip supplied!");
@@ -934,8 +937,8 @@ AVSValue GetProperty::Create(AVSValue args, void* user_data, IScriptEnvironment*
   int propType = (int)(intptr_t)user_data;
   // vUnset, vInt, vFloat, vData, vNode/Clip, vFrame/*, vMethod*/ }
   // 0: auto
-  // 1: integer
-  // 2: float
+  // 1: integer (int64 long)
+  // 2: float (double)
   // 3: char (null terminated data)
   // 4: Clip
 
@@ -972,7 +975,7 @@ AVSValue GetProperty::Create(AVSValue args, void* user_data, IScriptEnvironment*
   if (propType == 1) {
     int64_t result = env->propGetInt(avsmap, propName, index, &error);
     if(!error)
-      return AVSValue((int)result);
+      return AVSValue(result); // v11: no need for (int) cast
   }
   else if (propType == 2) {
     double result = env->propGetFloat(avsmap, propName, index, &error);
@@ -1011,8 +1014,11 @@ AVSValue GetProperty::Create(AVSValue args, void* user_data, IScriptEnvironment*
   return AVSValue();
 }
 
-AVSValue GetPropertyAsArray::Create(AVSValue args, void* , IScriptEnvironment* env)
+AVSValue GetPropertyAsArray::Create(AVSValue args, void* , IScriptEnvironment* env_)
 {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propGetAsArray: No clip supplied!");
@@ -1046,16 +1052,16 @@ AVSValue GetPropertyAsArray::Create(AVSValue args, void* , IScriptEnvironment* e
   std::vector<AVSValue> result(size);
 
   // propGetIntArray or propGetFloatArray is available
-  // note: AVSValue is int and float, prop arrays are int64_t and double
+  // Pre interface V11: AVSValue is int and float, prop arrays are int64_t and double
   if (propType == 'i') {
     const int64_t* arr = env->propGetIntArray(avsmap, propName, &error);
     for (int i = 0; i < size; ++i)
-      result[i] = (int)arr[i];
+      result[i] = arr[i]; // v11: no need for (int) cast
   }
   else if (propType == 'f') {
     const double* arr = env->propGetFloatArray(avsmap, propName, &error);
     for (int i = 0; i < size; ++i)
-      result[i] = (float)arr[i];
+      result[i] = arr[i]; // v11: no need for (float) cast
   }
   else
   {
@@ -1063,8 +1069,8 @@ AVSValue GetPropertyAsArray::Create(AVSValue args, void* , IScriptEnvironment* e
     for (int i = 0; i < size; ++i) {
       AVSValue elem;
       switch (propType) {
-      case 'i': elem = AVSValue((int)env->propGetInt(avsmap, propName, i, &error)); break; // though handled earlier
-      case 'f': elem = AVSValue((float)env->propGetFloat(avsmap, propName, i, &error)); break; // though handled earlier
+      case 'i': elem = AVSValue(env->propGetInt(avsmap, propName, i, &error)); break; // though handled earlier
+      case 'f': elem = AVSValue(env->propGetFloat(avsmap, propName, i, &error)); break; // though handled earlier
       case 's': {
         const char* s = env->propGetData(avsmap, propName, i, &error);
         if (!error)
@@ -1104,8 +1110,11 @@ AVSValue GetPropertyAsArray::Create(AVSValue args, void* , IScriptEnvironment* e
 // each array element is a two dimensional array [key, value]
 // value can be an array as well, depending on the property
 // only int, float and string data extracted
-AVSValue GetAllProperties::Create(AVSValue args, void*, IScriptEnvironment* env)
+AVSValue GetAllProperties::Create(AVSValue args, void*, IScriptEnvironment* env_)
 {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propGetAll: No clip supplied!");
@@ -1147,23 +1156,23 @@ AVSValue GetAllProperties::Create(AVSValue args, void*, IScriptEnvironment* env)
     }
     else if (propType == 'i') {
       if(propNumElements == 1)
-        elem = AVSValue((int)env->propGetInt(avsmap, propName, 0, &error));
+        elem = AVSValue(env->propGetInt(avsmap, propName, 0, &error)); // v11: no need for (int) cast
       else {
         std::vector<AVSValue> avsarr(propNumElements);
         const int64_t* arr = env->propGetIntArray(avsmap, propName, &error);
         for (int i = 0; i < propNumElements; ++i)
-          avsarr[i] = (int)arr[i];
+          avsarr[i] = arr[i]; // v11: no need for (int) cast
         elem = AVSValue(avsarr.data(), propNumElements); // array deep copy
       }
     }
     else if (propType == 'f') {
       if(propNumElements == 1)
-        elem = AVSValue((float)env->propGetFloat(avsmap, propName, 0, &error));
+        elem = AVSValue(env->propGetFloat(avsmap, propName, 0, &error)); // since v11 no need for (float) cast
       else {
         std::vector<AVSValue> avsarr(propNumElements);
         const double* arr = env->propGetFloatArray(avsmap, propName, &error);
         for (int i = 0; i < propNumElements; ++i)
-          avsarr[i] = (float)arr[i];
+          avsarr[i] = arr[i]; // v11: no need for (float) cast
         elem = AVSValue(avsarr.data(), propNumElements); // array deep copy
       }
     }
@@ -1195,7 +1204,10 @@ AVSValue GetAllProperties::Create(AVSValue args, void*, IScriptEnvironment* env)
 }
 
 // e.g. string length
-AVSValue GetPropertyDataSize::Create(AVSValue args, void* , IScriptEnvironment* env) {
+AVSValue GetPropertyDataSize::Create(AVSValue args, void* , IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propGetDataSize: No clip supplied!");
@@ -1240,7 +1252,10 @@ AVSValue GetPropertyDataSize::Create(AVSValue args, void* , IScriptEnvironment*
 }
 
 // array size of a given property, (by setProp append mode arrays can be constructed)
-AVSValue GetPropertyNumElements::Create(AVSValue args, void*, IScriptEnvironment* env) {
+AVSValue GetPropertyNumElements::Create(AVSValue args, void*, IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propNumElements: No clip supplied!");
@@ -1273,7 +1288,10 @@ AVSValue GetPropertyNumElements::Create(AVSValue args, void*, IScriptEnvironment
 }
 
 // returns integer enums instead of char (string)
-AVSValue GetPropertyType::Create(AVSValue args, void*, IScriptEnvironment* env) {
+AVSValue GetPropertyType::Create(AVSValue args, void*, IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propGetType: No clip supplied!");
@@ -1318,7 +1336,10 @@ AVSValue GetPropertyType::Create(AVSValue args, void*, IScriptEnvironment* env)
 
 
 // Number of properties (keys) for a frame
-AVSValue GetPropertyNumKeys::Create(AVSValue args, void*, IScriptEnvironment* env) {
+AVSValue GetPropertyNumKeys::Create(AVSValue args, void*, IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propNumKeys: No clip supplied!");
@@ -1349,7 +1370,10 @@ AVSValue GetPropertyNumKeys::Create(AVSValue args, void*, IScriptEnvironment* en
 }
 
 // Get the name (key) of the Nth frame property
-AVSValue GetPropertyKeyByIndex::Create(AVSValue args, void*, IScriptEnvironment* env) {
+AVSValue GetPropertyKeyByIndex::Create(AVSValue args, void*, IScriptEnvironment* env_) {
+  InternalEnvironment* IEnv = GetAndRevealCamouflagedEnv(env_);
+  IScriptEnvironment* env = static_cast<IScriptEnvironment*>(IEnv);
+
   AVSValue clip = args[0];
   if (!clip.IsClip())
     env->ThrowError("propNumKeys: No clip supplied!");
