@@ -1,5 +1,5 @@
 // Avisynth v2.5.
-// http://www.avisynth.org
+// http://avisynth.nl
 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include <vector>
 #include <avisynth.h>
 #include <avs/minmax.h>
-#include "SoundTouch/SoundTouch.h"
+#include <SoundTouch.h>
 
 
 #define BUFFERSIZE 8192
